/**
 * Simple demo class parsing a model and printing some information about the
 * model (SET, PARAMETERS, etc.) This demo uses the NativeLPL class.
 * @author  <a href="mailto:alexandre.gachet@unifr.ch">Alexandre Gachet</a>
 */

import lpl.*;
import java.util.StringTokenizer;

public class LPLC implements LPLCallback, LogCallback {    
    public static void main(String[] args) {
        LPLC demo = new LPLC();
        demo.compile();
        System.out.println("\n\n--------------demo calls ---------------\n\n");
        //demo.test();
    }

    //implement LPL callbacks
    public void callback() { ;} //{System.out.println(".");} 
    public void callback(String message) {System.out.println(message);}

    private void compile() {
      //NativeLPL.LPLinitParam();
      NativeLPL.LPLinit("alloy.lpl",0,0,0);
      NativeLPL.LPLcompileWithCallbacks("ww",this,this);
      // chech whether a default focus was set
      //if (NativeLPL.LPLisFocus()) {System.out.println("focus ok"); }
      //else {System.out.println("focus not ok"); }
      NativeLPL.LPLfree();
    }

    //further tests: do show this remove the comments in the main and compile again
    private void test() {
      System.out.println("You are using LPL version " + NativeLPL.LPLgetP(8) + 
                         " (" + NativeLPL.LPLgetP(16) + ")");
      System.out.println("LPL working path: " + NativeLPL.LPLgetP(6));
      NativeLPL.LPLinitParam();
      NativeLPL.LPLsetP(12, "");
      System.out.println("\n-------------------------------------------------");
      System.out.println("Trying to solve model allox.lpl (does not exist!)");
      System.out.println("-------------------------------------------------");
      NativeLPL.LPLinit("allox.lpl", 20000, 10000, 50000);
      System.out.println("State of the LPL compiler: " + NativeLPL.LPLwhere());
      System.out.println("Result of the compilation: " + NativeLPL.LPLcompile("vw"));
      int err = NativeLPL.LPLgetError();
      System.out.println("Last error of the compilation: " + err);
      System.out.println("Corresponding message: " + NativeLPL.LPLgetErrMessage(err));
      NativeLPL.LPLfree();
        
      System.out.println("\n-------------------------------");
      System.out.println("Trying to solve model alloy.lpl");
      System.out.println("-------------------------------");
      NativeLPL.LPLinit("alloy.lpl", 20000, 10000, 50000);
      System.out.println("Result of the compilation: " + NativeLPL.LPLcompile("vw"));

      System.out.println("\n-- outputs a tree---\n");
      NativeLPL.LPLsetFocus(NativeLPL.LPLgetHandle("metreq"));
      System.out.println(NativeLPL.LPLgetAttr(3));
      System.out.println("\n-------\n");
      if (NativeLPL.LPLisFocus()) { System.out.println(NativeLPL.LPLgetAttr(3));}
      System.out.println("\n--------------------\n"); 

      System.out.println("\nList of the SETs in the model (and their elements):");
      int i=0, j = 0;
      String nextSet = null, nextElement = null, element = null;
      StringTokenizer st = null;
      
      NativeLPL.LPLsetFirstEntity(1);
      while(NativeLPL.LPLisFocus()) {
        nextSet = NativeLPL.LPLgetAttr(4);
        if(nextSet.length() <= 0) break;
        System.out.println("   " + nextSet);
        j = 0;
        st = new StringTokenizer(nextSet,",");
        element = st.nextToken(); // the first token is the real name
        while(true) {
           nextElement = NativeLPL.LPLgetElementList(++j, 0);
           if(nextElement.length() <= 0) break;
           System.out.println("      " + nextElement);
        }
        NativeLPL.LPLnextEntity(1);
      }
      NativeLPL.LPLfree();
      System.out.println("\n[End of test]");
    } 

}
